@@ -25,4 +25,4 @@ minimizer = zfit.minimize.Minuit()
 result = minimizer.minimize(nll)
 
 # do the error calculations, here with minos
-param_errors = result.errors()
+param_errors, _ = result.errors()
