@@ -61,7 +61,23 @@ class TmpGaussian(zfit.pdf.BasePDF):
         params = {'mu': mu, 'sigma': sigma}
         super().__init__(obs, params, name=name, **kwargs)
 
-    def _unnormalized_pdf(self, x, norm_range=False):
+    def _unnormalized_pdf(self, x):
+        x = x.unstack_x()
+        mu = self.params['mu']
+        sigma = self.params['sigma']
+
+        return ztf.exp((-(x - mu) ** 2) / (
+            2 * sigma ** 2))  # non-normalized gaussian
+
+
+class TmpGaussianPDFNonNormed(zfit.pdf.BasePDF):
+
+    def __init__(self, obs, mu, sigma, params=None,
+                 name: str = "BasePDF", **kwargs):
+        params = {'mu': mu, 'sigma': sigma}
+        super().__init__(obs, params, name=name, **kwargs)
+
+    def _pdf(self, x, norm_range=False):
         x = x.unstack_x()
         mu = self.params['mu']
         sigma = self.params['sigma']
@@ -78,7 +94,29 @@ def create_test_gauss1():
     return test_gauss1, mu2, sigma2
 
 
-gaussian_dists = [lambda: create_gauss1(), lambda: create_test_gauss1()]
+def create_test_pdf_overriden_gauss1():
+    mu2 = zfit.Parameter("mu2_sampling1", mu_true, mu_true - 2., mu_true + 7.)
+    sigma2 = zfit.Parameter("sigma2_sampling1", sigma_true, sigma_true - 10., sigma_true + 5.)
+
+    test_gauss1 = TmpGaussianPDFNonNormed(name="test_pdf_nonnormed_gauss1", mu=mu2, sigma=sigma2, obs=obs1)
+    return test_gauss1, mu2, sigma2
+
+
+gaussian_dists = [create_gauss1, create_test_gauss1]
+
+
+def test_mutlidim_sampling():
+    zfit.run.set_graph_mode(False)
+    spaces = [zfit.Space(f'obs{i}', (i * 10, i * 10 + 6)) for i in range(4)]
+    pdfs = [GaussNoAnalyticSampling(obs=spaces[0], mu=3, sigma=1),
+            GaussNoAnalyticSampling(obs=spaces[2], mu=23, sigma=1),
+            UniformNoAnalyticSampling(obs=spaces[1], low=12, high=14),
+            UniformNoAnalyticSampling(obs=spaces[3], low=32, high=34),
+            ]
+    prod = zfit.pdf.ProductPDF(pdfs)
+    sample = prod.sample(n=20000)
+    for i, space in enumerate([p.space for p in pdfs]):
+        assert all(space.inside(sample.value()[:, i]))
 
 
 @pytest.mark.flaky(2)  # sampling
@@ -107,7 +145,7 @@ def test_multiple_limits_sampling(gauss_factory):
     assert np.std(sample2.value()) == pytest.approx(sigma_true, rel_tolerance)
 
 
-@pytest.mark.parametrize('gauss_factory', gaussian_dists)
+@pytest.mark.parametrize('gauss_factory', gaussian_dists + [create_test_pdf_overriden_gauss1])
 def test_sampling_fixed(gauss_factory):
     gauss, mu, sigma = gauss_factory()
 
@@ -257,7 +295,7 @@ def test_importance_sampling():
             thresholds = tf.random.uniform(shape=(n_to_produce,), dtype=dtype)
             return gaussian_sample, thresholds, weights, weights_max, n_to_produce
 
-    sample = accept_reject_sample(prob=gauss_pdf.unnormalized_pdf, n=30000, limits=obs_pdf)
+    sample = accept_reject_sample(prob=lambda x: gauss_pdf.pdf(x, norm_range=False), n=30000, limits=obs_pdf)
     gauss_pdf._sample_and_weights = GaussianSampleAndWeights
     sample2 = gauss_pdf.sample(n=30000, limits=obs_pdf)
     assert importance_sampling_called[0]
