@@ -30,9 +30,9 @@ class BaseConstraint(ZfitConstraint, BaseNumeric):
         """Base class for constraints.
 
         Args:
-            dtype (DType): the dtype of the constraint
-            name (str): the name of the constraint
-            params (Dict(str, :py:class:`~zfit.Parameter`)): A dictionary with the internal name of the
+            dtype: the dtype of the constraint
+            name: the name of the constraint
+            params: A dictionary with the internal name of the
                 parameter and the parameters itself the constrains depends on
         """
         super().__init__(name=name, dtype=dtype, params=params, **kwargs)
@@ -80,10 +80,10 @@ class ProbabilityConstraint(BaseConstraint):
         """Base class for constraints using a probability density function.
 
         Args:
-            dtype (DType): the dtype of the constraint
-            name (str): the name of the constraint
-            params (list(zfit.Parameter)): The parameters to constraint
-            observation (list(numerical) or list(zfit.Parameter)): Observed values of the parameter
+            dtype: the dtype of the constraint
+            name: the name of the constraint
+            params: The parameters to constraint
+            observation: Observed values of the parameter
                 to constraint obtained from auxiliary measurements.
         """
 
@@ -124,9 +124,8 @@ class ProbabilityConstraint(BaseConstraint):
         """Sample `n` points from the probability density function for the observed value of the parameters.
 
         Args:
-            n (int, tf.Tensor): The number of samples to be generated.
+            n: The number of samples to be generated.
         Returns:
-            Dict(Parameter: n_samples)
         """
         sample = self._sample(n=n)
         return {p: sample[:, i] for i, p in enumerate(self.observation)}
@@ -149,7 +148,7 @@ class TFProbabilityConstraint(ProbabilityConstraint):
         """ Base class for constraints using a probability density function from `tensorflow_probability`.
 
         Args:
-            distribution (`tensorflow_probability.distributions.Distribution`): The probability density function
+            distribution: The probability density function
                 used to constraint the parameters
 
         """
@@ -182,26 +181,25 @@ class GaussianConstraint(TFProbabilityConstraint):
 
     def __init__(self, params: ztyping.ParamTypeInput, observation: ztyping.NumericalScalarType,
                  uncertainty: ztyping.NumericalScalarType):
-        """Gaussian constraints on a list of parameters to some observed values with uncertainties.
+        r"""Gaussian constraints on a list of parameters to some observed values with uncertainties.
 
         A Gaussian constraint is defined as the likelihood of `params` given the `observations` and `uncertainty` from
         a different measurement.
 
-
         .. math::
-            constraint = Gauss(observation; params, uncertainty)
+            \text{constraint} = \text{Gauss}(\text{observation}; \text{params}, \text{uncertainty})
 
 
         Args:
-            params (list(zfit.Parameter)): The parameters to constraint; corresponds to mu in the Gaussian
+            params: The parameters to constraint; corresponds to x in the Gaussian
                 distribution.
-            observation (numerical, list(numerical)): observed values of the parameter; corresponds to the x argument
+            observation: observed values of the parameter; corresponds to mu
                 in the Gaussian distribution.
-            uncertainty (numerical, list(numerical) or array/tensor): Uncertainties or covariance/error
+            uncertainty: Uncertainties or covariance/error
                 matrix of the observed values. Can either be a single value, a list of values, an array or a tensor.
                 Corresponds to the sigma of the Gaussian distribution.
         Raises:
-            ShapeIncompatibleError: if params, mu and sigma don't have incompatible shapes
+            ShapeIncompatibleError: If params, mu and sigma have incompatible shapes.
         """
 
         observation = convert_to_container(observation, tuple)
