@@ -59,13 +59,13 @@ def limits_overlap(spaces: ztyping.SpaceOrSpacesTypeInput, allow_exact_match: bo
 
 
     Args:
-        spaces (Iterable[zfit.Space]):
-        allow_exact_match (bool): An exact overlap of two limits is counted as "not overlapping".
+        spaces:
+        allow_exact_match: An exact overlap of two limits is counted as "not overlapping".
             Example: limits from -1 to 3 and 4 to 5 to *NOT* overlap with the limits 4 to 5 *iff*
             `allow_exact_match` is True.
 
     Returns:
-        bool: if there are overlapping limits.
+        If there are overlapping limits.
     """
     # TODO(Mayou36): add approx comparison global in zfit
     eps = 1e-8  # epsilon for float comparisons
@@ -110,10 +110,10 @@ def common_obs(spaces: ztyping.SpaceOrSpacesTypeInput) -> Union[List[str], bool]
         returns ['obs1', 'obs3', 'obs2']
 
     Args:
-        spaces (): :py:class:`~zfit.Space`s to extract the obs from
+        spaces: :py:class:`~zfit.Space`s to extract the obs from
 
     Returns:
-        List[str]: The observables as `str` or False if not every space has observables
+        The observables as `str` or False if not every space has observables
     """
     spaces = convert_to_container(spaces, container=tuple)
     all_obs = []
@@ -137,10 +137,10 @@ def common_axes(spaces: ztyping.SpaceOrSpacesTypeInput) -> Union[List[str], bool
         returns [1, 3, 2]
 
     Args:
-        spaces (): :py:class:`~zfit.Space`s to extract the axes from
+        spaces: :py:class:`~zfit.Space`s to extract the axes from
 
     Returns:
-        List[int] or False: The axes as int or False if not every space has axes
+        The axes as int or False if not every space has axes
     """
     spaces = convert_to_container(spaces, container=tuple)
     all_axes = []
@@ -160,7 +160,7 @@ def obs_subsets(dimensionals: Iterable[ZfitDimensional]) -> Dict[Set[str], ZfitD
         dimensionals: An Iterable containing two or more ZfitDimensional that should be split into the smallest subset.
 
     Returns:
-        dict: dict with the keys being sets of observables and the values, an iterable, containing the ZfitDimensional
+        Dict with the keys being sets of observables and the values, an iterable, containing the ZfitDimensional
     """
     obs_dims = {}
     for dim in dimensionals:
