@@ -27,12 +27,12 @@ def _unbinned_nll_tf(model: ztyping.PDFInputType, data: ztyping.DataInputType, f
     """Return unbinned negative log likelihood graph for a PDF
 
     Args:
-        model (ZfitModel): PDFs with a `.pdf` method. Has to be as many models as data
-        data (ZfitData):
-        fit_range ():
+        model: PDFs with a `.pdf` method. Has to be as many models as data
+        data:
+        fit_range:
 
     Returns:
-        graph: the unbinned nll
+        The unbinned nll
 
     Raises:
         ValueError: if both `probs` and `log_probs` are specified.
@@ -86,12 +86,12 @@ class BaseLoss(ZfitLoss, BaseNumeric):
         to the loss. The length of each has to match the length of the others.
 
         Args:
-            model (Iterable[ZfitModel]): The model or models to evaluate the data on
-            data (Iterable[ZfitData]): Data to use
-            fit_range (Iterable[:py:class:`~zfit.Space`]): The fitting range. It's the norm_range for the models (if
+            model: The model or models to evaluate the data on
+            data: Data to use
+            fit_range: The fitting range. It's the norm_range for the models (if
             they
                 have a norm_range) and the data_range for the data.
-            constraints (Iterable[tf.Tensor): A Tensor representing a loss constraint. Using
+            constraints: A Tensor representing a loss constraint. Using
                 `zfit.constraint.*` allows for easy use of predefined constraints.
         """
         super().__init__(name=type(self).__name__, params={})
@@ -118,7 +118,7 @@ class BaseLoss(ZfitLoss, BaseNumeric):
                     is_yield: Optional[bool] = None,
                     extract_independent: Optional[bool] = True) -> Set["ZfitParameter"]:
         params = OrderedSet()
-        params = params.union(*(model.get_params(floating=floating, is_yield=False,
+        params = params.union(*(model.get_params(floating=floating, is_yield=is_yield,
                                                  extract_independent=extract_independent)
                                 for model in self.model))
 
@@ -174,11 +174,15 @@ class BaseLoss(ZfitLoss, BaseNumeric):
         self.add_cache_deps(cache_deps=fit_range)
         return pdf, data, fit_range
 
-    def gradients(self, params: ztyping.ParamTypeInput = None) -> List[tf.Tensor]:
+    def _input_check_params(self, params):
         if params is None:
-            params = list(self.get_cache_deps())
+            params = list(self.get_params())
         else:
             params = convert_to_container(params)
+        return params
+
+    def gradients(self, params: ztyping.ParamTypeInput = None) -> List[tf.Tensor]:
+        params = self._input_check_params(params)
         return self._gradients(params=params)
 
     def add_constraints(self, constraints):
@@ -255,6 +259,7 @@ class BaseLoss(ZfitLoss, BaseNumeric):
         return gradients
 
     def value_gradients(self, params: ztyping.ParamTypeInput) -> Tuple[tf.Tensor, tf.Tensor]:
+        params = self._input_check_params(params)
         return self._value_gradients(params=params)
 
     def _value_gradients(self, params):
@@ -266,6 +271,7 @@ class BaseLoss(ZfitLoss, BaseNumeric):
 
     def value_gradients_hessian(self, params: ztyping.ParamTypeInput, hessian=None) -> Tuple[
         tf.Tensor, tf.Tensor, tf.Tensor]:
+        params = self._input_check_params(params)
         numerical = settings.options['numerical_grad']
         vals = self._value_gradients_hessian(params=params, hessian=hessian, numerical=numerical)
 
@@ -353,6 +359,11 @@ class UnbinnedNLL(BaseLoss):
     def __init__(self, model, data, fit_range=None, constraints=None):
         super().__init__(model=model, data=data, fit_range=fit_range, constraints=constraints)
         self._errordef = 0.5
+        extended_pdfs = [pdf for pdf in self.model if pdf.is_extended]
+        if extended_pdfs and type(self) == UnbinnedNLL:
+            warn_advanced_feature("Extended PDFs are given to a normal UnbinnedNLL. This won't take the yield "
+                                  "into account and simply treat the PDFs as non-extended PDFs. To create an "
+                                  "extended NLL, use the `ExtendedUnbinnedNLL`.", identifier='extended_in_UnbinnedNLL')
 
     @z.function(wraps='loss')
     def _loss_func(self, model, data, fit_range, constraints):
@@ -360,6 +371,10 @@ class UnbinnedNLL(BaseLoss):
 
         return nll
 
+    @property
+    def is_extended(self):
+        return False
+
     @z.function(wraps='loss')
     def _loss_func_watched(self, constraints, data, fit_range, model):
         nll = _unbinned_nll_tf(model=model, data=data, fit_range=fit_range)
@@ -368,6 +383,12 @@ class UnbinnedNLL(BaseLoss):
             nll += constraints
         return nll
 
+    def _get_params(self, floating: Optional[bool] = True, is_yield: Optional[bool] = None,
+                    extract_independent: Optional[bool] = True) -> Set["ZfitParameter"]:
+        if not self.is_extended:
+            is_yield = False  # the loss does not depend on the yields
+        return super()._get_params(floating, is_yield, extract_independent)
+
     # def _cache_add_constraints(self, constraints):
     #     if self._cache.get('loss') is not None:
     #         constraints = [c.value() for c in constraints]
@@ -396,6 +417,10 @@ class ExtendedUnbinnedNLL(UnbinnedNLL):
         nll += tf.reduce_sum(term_new, axis=0)
         return nll
 
+    @property
+    def is_extended(self):
+        return True
+
 
 class SimpleLoss(BaseLoss):
     _name = "SimpleLoss"
