@@ -3,24 +3,26 @@ import contextlib
 import itertools
 import warnings
 from collections import OrderedDict
-from typing import Dict, Union, Callable, Optional, Tuple
+from typing import Dict, Union, Callable, Optional, Tuple, Iterable
 
 import colored
+import iminuit
 import numpy as np
-from colorama import Style
+from colorama import Style, init
 from ordered_set import OrderedSet
 from tabulate import tabulate
 
-from .errors import compute_errors
+from .errors import compute_errors, covariance_with_weights, dict_to_matrix, matrix_to_dict
 from .interface import ZfitMinimizer, ZfitResult
 from ..core.interfaces import ZfitLoss, ZfitParameter
 from ..core.parameter import set_values
 from ..settings import run
 from ..util.container import convert_to_container
-from ..util.exception import WeightsNotImplementedError
 from ..util.warnings import ExperimentalFeatureWarning
 from ..util.ztyping import ParamsTypeOpt
 
+init(autoreset=True)
+
 
 def _minos_minuit(result, params, sigma=1.0):
     fitresult = result
@@ -39,9 +41,9 @@ def _minos_minuit(result, params, sigma=1.0):
 
 
 def _covariance_minuit(result, params):
-    # check if no weights in data
-    if any([data.weights is not None for data in result.loss.data]):
-        raise WeightsNotImplementedError("Weights are not supported with minuit hesse.")
+    if any(data.weights is not None for data in result.loss.data):
+        warnings.warn("The computation of the covariance matrix with weights is still experimental.",
+                      ExperimentalFeatureWarning)
 
     fitresult = result
     minimizer = fitresult.minimizer
@@ -64,15 +66,18 @@ def _covariance_minuit(result, params):
 
 
 def _covariance_np(result, params):
-    # check if no weights in data
-    if any([data.weights is not None for data in result.loss.data]):
-        raise WeightsNotImplementedError("Weights are not supported with hesse numpy.")
+    if any(data.weights is not None for data in result.loss.data):
+        warnings.warn("The computation of the covariance matrix with weights is still experimental.",
+                      ExperimentalFeatureWarning)
 
     # TODO: maybe activate again? currently fails due to numerical problems
     # numgrad_was_none = settings.options.numerical_grad is None
     # if numgrad_was_none:
     #     settings.options.numerical_grad = True
-    covariance = np.linalg.inv(result.loss.value_gradients_hessian(params)[2])
+
+    _, gradient, hessian = result.loss.value_gradients_hessian(params)
+    covariance = np.linalg.inv(hessian)
+
     # if numgrad_was_none:
     #     settings.options.numerical_grad = None
 
@@ -89,21 +94,21 @@ class FitResult(ZfitResult):
                  info: dict, loss: ZfitLoss, minimizer: "ZfitMinimizer"):
         """Create a `FitResult` from a minimization. Store parameter values, minimization infos and calculate errors.
 
-        Any errors calculated are saved under `self.params` dictionary with {parameter: {error_name1: {'low': value
-        'high': value or similar}}
+        Any errors calculated are saved under `self.params` dictionary with::
+
+            {parameter: {error_name1: {'low': value, 'high': value or similar}}
 
         Args:
-            params (OrderedDict[:py:class:`~zfit.Parameter`, float]): Result of the fit where each
-            :py:class:`~zfit.Parameter` key has the value
-                from the minimum found by the minimizer.
-            edm (Union[int, float]): The estimated distance to minimum, estimated by the minimizer (if available)
-            fmin (Union[numpy.float64, float]): The minimum of the function found by the minimizer
-            status (int): A status code (if available)
-            converged (bool): Whether the fit has successfully converged or not.
-            info (Dict): Additional information (if available) like *number of function calls* and the
+            params: Result of the fit where each
+               :py:class:`~zfit.Parameter` key has the value from the minimum found by the minimizer.
+            edm: The estimated distance to minimum, estimated by the minimizer (if available)
+            fmin: The minimum of the function found by the minimizer
+            status: A status code (if available)
+            converged: Whether the fit has successfully converged or not.
+            info: Additional information (if available) like *number of function calls* and the
                 original minimizer return message.
-            loss (Union[ZfitLoss]): The loss function that was minimized. Contains also the pdf, data etc.
-            minimizer (ZfitMinimizer): Minimizer that was used to obtain this `FitResult` and will be used to
+            loss: The loss function that was minimized. Contains also the pdf, data etc.
+            minimizer: Minimizer that was used to obtain this `FitResult` and will be used to
                 calculate certain errors. If the minimizer is state-based (like "iminuit"), then this is a copy
                 and the state of other `FitResults` or of the *actual* minimizer that performed the minimization
                 won't be altered.
@@ -120,14 +125,55 @@ class FitResult(ZfitResult):
         self._loss = loss
         self._minimizer = minimizer
         self._valid = True
+        self._covariance_dict = {}
 
     def _input_convert_params(self, params):
-        params = ParamHolder((p, {"value": v}) for p, v in params.items())
-        return params
+        return ParamHolder((p, {"value": v}) for p, v in params.items())
 
     def _get_uncached_params(self, params, method_name):
-        params_uncached = [p for p in params if self.params[p].get(method_name) is None]
-        return params_uncached
+        return [p for p in params if self.params[p].get(method_name) is None]
+
+    @classmethod
+    def from_minuit(cls, loss: ZfitLoss, params: Iterable[ZfitParameter], result: iminuit.util.MigradResult,
+                    minimizer: Union[ZfitMinimizer, iminuit.Minuit]) -> 'FitResult':
+        """Create a `FitResult` from a :py:class:~`iminuit.util.MigradResult` returned by
+        :py:meth:`iminuit.Minuit.migrad` and a iminuit :py:class:~`iminuit.Minuit` instance with the corresponding
+        zfit objects.
+
+        Args:
+            loss: zfit Loss that was minimized.
+            params: Iterable of the zfit parameters that were floating during the minimization.
+            result: Return value of the iminuit migrad command.
+            minimizer: Instance of the iminuit Minuit that was used to minimize the loss.
+
+        Returns:
+            A `FitResult` as if zfit Minuit was used.
+        """
+
+        from .minimizer_minuit import Minuit
+        if not isinstance(minimizer, Minuit):
+            if isinstance(minimizer, iminuit.Minuit):
+                minimizer_new = Minuit()
+                minimizer_new._minuit_minimizer = minimizer
+                minimizer = minimizer_new
+            else:
+                raise ValueError(f"Minimizer {minimizer} not supported. Use `Minuit` from zfit or from iminuit.")
+        params_result = [p_dict for p_dict in result[1]]
+        result_vals = [res["value"] for res in params_result]
+        set_values(params, values=result_vals)
+        info = {'n_eval': result[0]['nfcn'],
+                'n_iter': result[0]['ncalls'],
+                # 'grad': result['jac'],
+                # 'message': result['message'],
+                'original': result[0]}
+        edm = result[0]['edm']
+        fmin = result[0]['fval']
+        status = -999
+        converged = result[0]['is_valid']
+        params = OrderedDict((p, res['value']) for p, res in zip(params, params_result))
+        return cls(params=params, edm=edm, fmin=fmin, info=info, loss=loss,
+                   status=status, converged=converged,
+                   minimizer=minimizer)
 
     @property
     def params(self):
@@ -138,10 +184,9 @@ class FitResult(ZfitResult):
         """The estimated distance to the minimum.
 
         Returns:
-            numeric
+            Numeric
         """
-        edm = self._edm
-        return edm
+        return self._edm
 
     @property
     def minimizer(self):
@@ -157,15 +202,13 @@ class FitResult(ZfitResult):
         """Function value at the minimum.
 
         Returns:
-            numeric
+            Numeric
         """
-        fmin = self._fmin
-        return fmin
+        return self._fmin
 
     @property
     def status(self):
-        status = self._status
-        return status
+        return self._status
 
     @property
     def info(self):
@@ -208,13 +251,13 @@ class FitResult(ZfitResult):
         """Calculate for `params` the symmetric error using the Hessian/covariance matrix.
 
         Args:
-            params (list(:py:class:`~zfit.Parameter`)): The parameters  to calculate the
+            params: The parameters to calculate the
                 Hessian symmetric error. If None, use all parameters.
-            method (str): the method to calculate the covariance matrix. Can be {'minuit_hesse', 'hesse_np'} or a callable.
-            error_name (str): The name for the error in the dictionary.
+            method: the method to calculate the covariance matrix. Can be {'minuit_hesse', 'hesse_np'} or a callable.
+            error_name: The name for the error in the dictionary.
 
         Returns:
-            OrderedDict: Result of the hessian (symmetric) error as dict with each parameter holding
+            Result of the hessian (symmetric) error as dict with each parameter holding
                 the error dict {'error': sym_error}.
 
                 So given param_a (from zfit.Parameter(.))
@@ -234,16 +277,14 @@ class FitResult(ZfitResult):
                 raise ValueError("Need to specify `error_name` or use a string as `method`")
             error_name = method
 
-        all_params = list(self.params.keys())
-        uncached_params = self._get_uncached_params(params=all_params, method_name=error_name)
-
         with self._input_check_reset_params(params) as params:
-            if uncached_params:  # TODO: this logic doesn't make sense?
+            uncached_params = self._get_uncached_params(params=params, method_name=error_name)
+            if uncached_params:
                 error_dict = self._hesse(params=uncached_params, method=method)
                 self._cache_errors(error_name=error_name, errors=error_dict)
 
-        all_errors = OrderedDict((p, self.params[p][error_name]) for p in params)
-        return all_errors
+        errors = OrderedDict((p, self.params[p][error_name]) for p in params)
+        return errors
 
     def _cache_errors(self, error_name, errors):
         for param, errors in errors.items():
@@ -255,27 +296,31 @@ class FitResult(ZfitResult):
 
     def error(self, params: ParamsTypeOpt = None, method: Union[str, Callable] = None, error_name: str = None,
               sigma: float = 1.0) -> OrderedDict:
-        r"""DEPRECATED! Use 'errors' instead
-            Args:
-                params (list(:py:class:`~zfit.Parameter` or str)): The parameters or their names to calculate the
-                     errors. If `params` is `None`, use all *floating* parameters.
-                method (str or Callable): The method to use to calculate the errors. Valid choices are
-                    {'minuit_minos'} or a Callable.
-                sigma (float): Errors are calculated with respect to `sigma` std deviations. The definition
-                    of 1 sigma depends on the loss function and is defined there.
+        r"""
 
-                    For example, the negative log-likelihood (without the factor of 2) has a correspondents
-                    of :math:`\Delta` NLL of 1 corresponds to 1 std deviation.
-                error_name (str): The name for the error in the dictionary.
+        .. deprecated:: unknown
+            Use :func:`errors` instead.
+
+        Args:
+            params: The parameters or their names to calculate the
+                 errors. If `params` is `None`, use all *floating* parameters.
+            method: The method to use to calculate the errors. Valid choices are
+                {'minuit_minos'} or a Callable.
+            sigma: Errors are calculated with respect to `sigma` std deviations. The definition
+                of 1 sigma depends on the loss function and is defined there.
+
+                For example, the negative log-likelihood (without the factor of 2) has a correspondents
+                of :math:`\Delta` NLL of 1 corresponds to 1 std deviation.
+            error_name: The name for the error in the dictionary.
 
 
-            Returns:
-                `OrderedDict`: A `OrderedDict` containing as keys the parameter names and as value a `dict` which
-                    contains (next to probably more things) two keys 'lower' and 'upper',
-                    holding the calculated errors.
-                    Example: result['par1']['upper'] -> the asymmetric upper error of 'par1'
+        Returns:
+            A `OrderedDict` containing as keys the parameter names and as value a `dict` which
+                contains (next to probably more things) two keys 'lower' and 'upper',
+                holding the calculated errors.
+                Example: result['par1']['upper'] -> the asymmetric upper error of 'par1'
         """
-        warnings.warn("`error` is depreceated, use `errors` instead. This will return not only the errors but also "
+        warnings.warn("`error` is deprecated, use `errors` instead. This will return not only the errors but also "
                       "(a possible) new FitResult if a minimum was found. So change"
                       "errors = result.error()"
                       "to"
@@ -287,20 +332,20 @@ class FitResult(ZfitResult):
         r"""Calculate and set for `params` the asymmetric error using the set error method.
 
             Args:
-                params (list(:py:class:`~zfit.Parameter` or str)): The parameters or their names to calculate the
+                params: The parameters or their names to calculate the
                      errors. If `params` is `None`, use all *floating* parameters.
-                method (str or Callable): The method to use to calculate the errors. Valid choices are
+                method: The method to use to calculate the errors. Valid choices are
                     {'minuit_minos'} or a Callable.
-                sigma (float): Errors are calculated with respect to `sigma` std deviations. The definition
+                sigma: Errors are calculated with respect to `sigma` std deviations. The definition
                     of 1 sigma depends on the loss function and is defined there.
 
                     For example, the negative log-likelihood (without the factor of 2) has a correspondents
                     of :math:`\Delta` NLL of 1 corresponds to 1 std deviation.
-                error_name (str): The name for the error in the dictionary.
+                error_name: The name for the error in the dictionary.
 
 
             Returns:
-                `OrderedDict`: A `OrderedDict` containing as keys the parameter and as value a `dict` which
+                A `OrderedDict` containing as keys the parameter and as value a `dict` which
                     contains (next to probably more things) two keys 'lower' and 'upper',
                     holding the calculated errors.
                     Example: result[par1]['upper'] -> the asymmetric upper error of 'par1'
@@ -353,11 +398,11 @@ class FitResult(ZfitResult):
         """Calculate the covariance matrix for `params`.
 
             Args:
-                params (list(:py:class:`~zfit.Parameter`)): The parameters to calculate
+                params: The parameters to calculate
                     the covariance matrix. If `params` is `None`, use all *floating* parameters.
-                method (str or Callbel): The method to use to calculate the covariance matrix. Valid choices are
+                method: The method to use to calculate the covariance matrix. Valid choices are
                     {'minuit_hesse', 'hesse_np'} or a Callable.
-                as_dict (bool): Default `False`. If `True` then returns a dictionnary.
+                as_dict: Default `False`. If `True` then returns a dictionnary.
 
             Returns:
                 2D `numpy.array` of shape (N, N);
@@ -372,9 +417,12 @@ class FitResult(ZfitResult):
                 method = "minuit_hesse"
             # LEGACY END
 
-        with self._input_check_reset_params(params) as params:
-            covariance = self._covariance(method=method)
-        covariance = {k: covariance[k] for k in itertools.product(params, params)}
+        if method not in self._covariance_dict:
+            with self._input_check_reset_params(params) as params:
+                self._covariance_dict[method] = self._covariance(method=method)
+
+        params = self._input_check_params(params)
+        covariance = {k: self._covariance_dict[method][k] for k in itertools.product(params, params)}
 
         if as_dict:
             return covariance
@@ -389,15 +437,46 @@ class FitResult(ZfitResult):
                 raise KeyError("The following method is not a valid, implemented method: {}".format(method))
 
         params = list(self.params.keys())
-        return method(result=self, params=params)
+
+        if any([data.weights is not None for data in self.loss.data]):
+            return covariance_with_weights(method=method, result=self, params=params)
+        else:
+            return method(result=self, params=params)
+
+    def correlation(self, params: ParamsTypeOpt = None, method: Union[str, Callable] = None, as_dict: bool = False):
+        """Calculate the correlation matrix for `params`.
+
+            Args:
+                params: The parameters to calculate
+                    the correlation matrix. If `params` is `None`, use all *floating* parameters.
+                method: The method to use to calculate the correlation matrix. Valid choices are
+                    {'minuit_hesse', 'hesse_np'} or a Callable.
+                as_dict: Default `False`. If `True` then returns a dictionnary.
+
+            Returns:
+                2D `numpy.array` of shape (N, N);
+                `dict`(param1, param2) -> correlation if `as_dict == True`.
+        """
+
+        covariance = self.covariance(params=params, method=method, as_dict=False)
+        correlation = covariance_to_correlation(covariance)
+
+        if as_dict:
+            params = self._input_check_params(params)
+            return matrix_to_dict(params, correlation)
+        else:
+            return correlation
 
     def __str__(self):
         string = Style.BRIGHT + f'FitResult' + Style.NORMAL + f' of\n{self.loss} \nwith\n{self.minimizer}\n\n'
         string += tabulate(
-            [[color_on_bool(self.converged), format_value(self.edm, highprec=False),
+            [[color_on_bool(self.valid), color_on_bool(self.converged, on_true=False),
+              color_on_bool(self.params_at_limit, on_true=colored.bg(9), on_false=False),
+              format_value(self.edm, highprec=False),
               format_value(self.fmin)]],
-            ['converged', 'edm', 'min value'], tablefmt='fancy_grid'
-        )
+            ['valid', 'converged', 'param at limit', 'edm', 'min value'],
+            tablefmt='fancy_grid',
+            disable_numparse=True)
         string += '\n\n' + Style.BRIGHT + "Parameters\n" + Style.NORMAL
         string += str(self.params)
         return string
@@ -409,34 +488,9 @@ class FitResult(ZfitResult):
         p.text(self.__str__())
 
 
-def dict_to_matrix(params, matrix_dict):
-    nparams = len(params)
-    matrix = np.empty((nparams, nparams))
-
-    for i in range(nparams):
-        pi = params[i]
-        for j in range(i, nparams):
-            pj = params[j]
-            matrix[i, j] = matrix_dict[(pi, pj)]
-            if i != j:
-                matrix[j, i] = matrix_dict[(pi, pj)]
-
-    return matrix
-
-
-def matrix_to_dict(params, matrix):
-    nparams = len(params)
-    matrix_dict = {}
-
-    for i in range(nparams):
-        pi = params[i]
-        for j in range(i, nparams):
-            pj = params[j]
-            matrix_dict[(pi, pj)] = matrix[i, j]
-            if i != j:
-                matrix_dict[(pj, pi)] = matrix[i, j]
-
-    return matrix_dict
+def covariance_to_correlation(covariance):
+    diag = np.diag(1 / np.diag(covariance) ** 0.5)
+    return np.matmul(diag, np.matmul(covariance, diag))
 
 
 def format_value(value, highprec=True):
@@ -448,13 +502,18 @@ def format_value(value, highprec=True):
     if isinstance(value, (dict, m_error_class)):
         if 'error' in value:
             value = value['error']
-            value = f"+/- {value:> 6.2g}"
+            value = f"{value:> 6.2g}"
+            value = f'+/-{" " * (8 - len(value))}' + value
         if 'lower' in value and 'upper' in value:
             lower = value['lower']
             upper = value['upper']
-            lower, upper = f"{lower: >+6.2g}", f"{upper: >+6.2g}"
-            lower += " " * (9 - len(lower))
-            value = lower + upper
+            lower_sign = f"{np.sign(lower): >+}"[0]
+            upper_sign = f"{np.sign(upper): >+}"[0]
+            lower, upper = f"{np.abs(lower): >6.2g}", f"{upper: >6.2g}"
+            lower = lower_sign + " " * (7 - len(lower)) + lower
+            upper = upper_sign + " " * (7 - len(upper)) + upper
+            # lower += " t" * (11 - len(lower))
+            value = lower + " " * 3 + upper
 
     if isinstance(value, float):
         if highprec:
@@ -471,7 +530,7 @@ def color_on_bool(value, on_true=colored.bg(10), on_false=colored.bg(9)):
         value_add = on_true
     else:
         value_add = ''
-    value = value_add + str(value) + colored.bg(15)
+    value = value_add + str(value) + Style.RESET_ALL
     return value
 
 
@@ -493,5 +552,5 @@ class ParamHolder(dict):  # no UserDict, we only want to change the __str__
             rows.append(row)
 
         order_keys = ['name'] + list(order_keys) + ['at limit']
-        table = tabulate(rows, order_keys)
+        table = tabulate(rows, order_keys, numalign="right", stralign='right', colalign=('left',))
         return table
