@@ -1,18 +1,22 @@
-#  Copyright (c) 2019 zfit
+#  Copyright (c) 2020 zfit
 
 from .core.basepdf import BasePDF
 from .models.basic import Exponential
-from .models.physics import CrystalBall, DoubleCB
-from .models.dist_tfp import Gauss, Uniform, WrapDistribution, TruncatedGauss
-from .models.polynomials import Chebyshev, Legendre, Chebyshev2, Hermite, Laguerre, RecursivePolynomial
+from .models.dist_tfp import Gauss, Uniform, WrapDistribution, TruncatedGauss, Cauchy, Poisson
 from .models.functor import ProductPDF, SumPDF, BaseFunctor
+from .models.kde import GaussianKDE1DimV1
+from .models.physics import CrystalBall, DoubleCB
+from .models.polynomials import Chebyshev, Legendre, Chebyshev2, Hermite, Laguerre, RecursivePolynomial
 from .models.special import ZPDF, SimplePDF, SimpleFunctorPDF
+from .models.convolution import FFTConvPDFV1
 
 __all__ = ['BasePDF', 'BaseFunctor',
            'Exponential',
            'CrystalBall', 'DoubleCB',
-           'Gauss', 'Uniform', 'TruncatedGauss', 'WrapDistribution',
+           'Gauss', 'Uniform', 'TruncatedGauss', 'WrapDistribution', 'Cauchy', 'Poisson',
            "Chebyshev", "Legendre", "Chebyshev2", "Hermite", "Laguerre", "RecursivePolynomial",
            'ProductPDF', 'SumPDF',
+           'GaussianKDE1DimV1',
+           'FFTConvPDFV1',
            'ZPDF', 'SimplePDF', 'SimpleFunctorPDF'
            ]
