@@ -6,6 +6,8 @@ from typing import Iterable, Callable, Optional
 import numdifftools
 import tensorflow as tf
 
+from . import function
+from .tools import _auto_upcast
 from ..settings import ztypes
 from ..util.container import convert_to_container
 
@@ -14,11 +16,11 @@ def poly_complex(*args, real_x=False):
     """Complex polynomial with the last arg being x.
 
     Args:
-        *args (tf.Tensor or equ.): Coefficients of the polynomial
-        real_x (bool): If True, x is assumed to be real.
+        *args: Coefficients of the polynomial
+        real_x: If True, x is assumed to be real.
 
     Returns:
-        tf.Tensor:
+
     """
     from .. import z
 
@@ -35,11 +37,11 @@ def interpolate(t, c):
     """Multilinear interpolation on a rectangular grid of arbitrary number of dimensions.
 
     Args:
-        t (tf.Tensor): Grid (of rank N)
-        c (tf.Tensor): Tensor of coordinates for which the interpolation is performed
+        t: Grid (of rank N)
+        c: Tensor of coordinates for which the interpolation is performed
 
     Returns:
-        tf.Tensor: 1D tensor of interpolated value
+        1D tensor of interpolated value
     """
     rank = len(t.get_shape())
     ind = tf.cast(tf.floor(c), tf.int32)
@@ -58,12 +60,12 @@ def numerical_gradient(func: Callable, params: Iterable["zfit.Parameter"]) -> tf
     """Calculate numerically the gradients of func() with respect to `params`.
 
     Args:
-        func (Callable): Function without arguments that depends on `params`
-        params (ZfitParameter): Parameters that `func` implicitly depends on and with respect to which the
+        func: Function without arguments that depends on `params`
+        params: Parameters that `func` implicitly depends on and with respect to which the
             derivatives will be taken.
 
     Returns:
-        `tf.Tensor`: gradients
+        Gradients
     """
     params = convert_to_container(params)
 
@@ -89,12 +91,12 @@ def numerical_value_gradients(func: Callable, params: Iterable["zfit.Parameter"]
     """Calculate numerically the gradients of `func()` with respect to `params`, also returns the value of `func()`.
 
         Args:
-            func (Callable): Function without arguments that depends on `params`
-            params (ZfitParameter): Parameters that `func` implicitly depends on and with respect to which the
+            func: Function without arguments that depends on `params`
+            params: Parameters that `func` implicitly depends on and with respect to which the
                 derivatives will be taken.
 
         Returns:
-            tuple(`tf.Tensor`, `tf.Tensor`): value, gradient
+            Value, gradient
     """
     return func(), numerical_gradient(func, params)
 
@@ -103,12 +105,12 @@ def numerical_hessian(func: Callable, params: Iterable["zfit.Parameter"], hessia
     """Calculate numerically the hessian matrix of func with respect to `params`.
 
         Args:
-            func (Callable): Function without arguments that depends on `params`
-            params (ZfitParameter): Parameters that `func` implicitly depends on and with respect to which the
+            func: Function without arguments that depends on `params`
+            params: Parameters that `func` implicitly depends on and with respect to which the
                 derivatives will be taken.
 
         Returns:
-            `tf.Tensor`: hessian matrix
+            Hessian matrix
     """
     params = convert_to_container(params)
 
@@ -147,12 +149,12 @@ def numerical_value_gradients_hessian(func: Callable, params: Iterable["zfit.Par
     """Calculate numerically the gradients and hessian matrix of `func()` wrt `params`; also return `func()`.
 
         Args:
-            func (Callable): Function without arguments that depends on `params`
-            params (ZfitParameter): Parameters that `func` implicitly depends on and with respect to which the
+            func: Function without arguments that depends on `params`
+            params: Parameters that `func` implicitly depends on and with respect to which the
                 derivatives will be taken.
 
         Returns:
-            tuple(`tf.Tensor`, `tf.Tensor`, `tf.Tensor`): value, gradient and hessian matrix
+            Value, gradient and hessian matrix
     """
     value, gradients = numerical_value_gradients(func, params)
     hessian = numerical_hessian(func, params, hessian=hessian)
@@ -168,12 +170,12 @@ def autodiff_gradient(func: Callable, params: Iterable["zfit.Parameter"]) -> tf.
     TensorFlow implements this and anything using `tf.*` operations only can use this technique.
 
         Args:
-            func (Callable): Function without arguments that depends on `params`
-            params (ZfitParameter): Parameters that `func` implicitly depends on and with respect to which the
+            func: Function without arguments that depends on `params`
+            params: Parameters that `func` implicitly depends on and with respect to which the
                 derivatives will be taken.
 
         Returns:
-            `tf.Tensor`: gradient
+            Gradient
     """
     return autodiff_value_gradients(func, params)[1]
 
@@ -186,12 +188,12 @@ def autodiff_value_gradients(func: Callable, params: Iterable["zfit.Parameter"])
     TensorFlow implements this and anything using `tf.*` operations only can use this technique.
 
         Args:
-            func (Callable): Function without arguments that depends on `params`
-            params (ZfitParameter): Parameters that `func` implicitly depends on and with respect to which the
+            func: Function without arguments that depends on `params`
+            params: Parameters that `func` implicitly depends on and with respect to which the
                 derivatives will be taken.
 
         Returns:
-            tuple(`tf.Tensor`, `tf.Tensor`): value and gradient
+            Value and gradient
     """
     with tf.GradientTape(persistent=False,  # needs to be persistent for a call from hessian.
                          watch_accessed_variables=False) as tape:
@@ -209,12 +211,12 @@ def autodiff_hessian(func: Callable, params: Iterable["zfit.Parameter"], hessian
     TensorFlow implements this and anything using `tf.*` operations only can use this technique.
 
         Args:
-            func (Callable): Function without arguments that depends on `params`
-            params (ZfitParameter): Parameters that `func` implicitly depends on and with respect to which the
+            func: Function without arguments that depends on `params`
+            params: Parameters that `func` implicitly depends on and with respect to which the
                 derivatives will be taken.
 
         Returns:
-            `tf.Tensor`: hessian matrix
+            Hessian matrix
     """
 
     return automatic_value_gradients_hessian(func, params, hessian=hessian)[2]
@@ -230,12 +232,12 @@ def automatic_value_gradients_hessian(func: Callable = None, params: Iterable["z
     TensorFlow implements this and anything using `tf.*` operations only can use this technique.
 
         Args:
-            func (Callable): Function without arguments that depends on `params`
-            params (ZfitParameter): Parameters that `func` implicitly depends on and with respect to which the
+            func: Function without arguments that depends on `params`
+            params: Parameters that `func` implicitly depends on and with respect to which the
                 derivatives will be taken.
 
         Returns:
-            tuple(`tf.Tensor`, `tf.Tensor`, `tf.Tensor`): value, gradient and hessian matrix
+            Value, gradient and hessian matrix
     """
     if params is None:
         raise ValueError("Parameters have to be specified, are currently None.")
@@ -264,3 +266,14 @@ def automatic_value_gradients_hessian(func: Callable = None, params: Iterable["z
                                                              ))
     del tape
     return loss, gradients, computed_hessian
+
+
+@function(wraps="tensor")
+def reduce_geometric_mean(input_tensor, axis=None, keepdims=False):
+    log_mean = tf.reduce_mean(tf.math.log(input_tensor), axis=axis, keepdims=keepdims)
+    return tf.math.exp(log_mean)
+
+
+def log(x, name=None):
+    x = _auto_upcast(x)
+    return _auto_upcast(tf.math.log(x=x, name=name))
